#version 120
#extension GL_EXT_gpu_shader4 : enable

float luma_mod;
vec3 avg_luma_vec;

// these need to stay up at the top apparently, to avoid some
// fun compilation bugs
uniform sampler2D RTScene;
uniform float rt_w;
uniform float rt_h;
//uniform vec2 rcpFrame;
noperspective varying vec2 pos;

// For all settings: 1.0 = 100% 0.5=50% 1.5 = 150%
vec3 BrightnessSaturationContrast(vec3 color, float brt, float sat, float con)
{
	// Increase or decrease theese values to adjust r, g and b color channels seperately
	/*const*/ float AvgLumR = 0.0; //avg_luma_vec.r; //0.5;
	/*const*/ float AvgLumG = 0.72; //avg_luma_vec.g; //0.5;
	/*const*/ float AvgLumB = 0.7; //avg_luma_vec.b; //0.5;
	
	const vec3 LumCoeff = vec3(0.2125, 0.7154, 0.0721);
	
	vec3 AvgLumin = vec3(AvgLumR, AvgLumG, AvgLumB);
	vec3 brtColor = color * brt;
	vec3 intensity = vec3(dot(brtColor, LumCoeff));
	vec3 satColor = mix(intensity, brtColor, sat);
	//vec3 conColor = mix((intensity - AvgLumin) / AvgLumin, satColor, con); 
	
	vec3 conColor = mix(mix(AvgLumin, intensity, 0.5) /*intensity*/, satColor, con);
	return conColor;
}

// bleach bypass
vec3 overlay(vec3 previousmix, vec3 amount)
{
    const vec3 lumcoeff = vec3(0.2125, 0.7154, 0.0721);
    const vec3 one = vec3(1.0);	
    const vec3 two = vec3(2.0);
    vec3 myInput = vec3(dot(previousmix, lumcoeff));
    
	float luminance = dot(previousmix,lumcoeff);
	float mixamount = clamp((luminance - 0.45) * 10.0, 0.0, 1.0);

	vec3 branch1 = two * previousmix * myInput;
	vec3 branch2 = one - (two * (one - previousmix) * (one - myInput));
	
	vec3 result = mix(branch1, branch2, vec3(mixamount));

	return mix(previousmix, result, amount);
}

vec4 bypass(vec4 base)
{    
    vec3 lumCoeff = vec3(0.25,0.65,0.1); //vec3(0.25,0.65,0.1);
    float lum = dot(lumCoeff,base.rgb);
    vec3 blend = vec3(lum);
    float L = min(1,max(0,10.0*(lum- 0.45)));
    vec3 result1 = 2.0f * base.rgb * blend; 
    vec3 result2 = 1.0f - 2.0f*(1.0f-blend)*(1.0f-base.rgb);
    vec3 newColor = mix(result1,result2,L); // was lerp
    float A2 = /*Opacity*/ 2.0 * base.a;
    vec3 mixRGB = A2 * newColor.rgb; 
    mixRGB += ((1.0f-A2) * base.rgb); 
    return vec4(mixRGB,base.a);
}

void main() 
{ 
    vec2 rcpFrame = vec2(1.0 / rt_w, 1.0 / rt_h); //vec2(1.0 / rt_w, 1.0 / rt_h);
        
    vec3 fxaa_color = texture2D(RTScene, pos).xyz;    

    //**********
    // "overlay" is a bleach bypass, a la Saving Private Ryan or Black Hawk Down
    //**********

    fxaa_color = overlay(fxaa_color, vec3(0.80));    // 0.33 // 0.75

    //**********
    // sepia tones, if you like that sort of thing!
    //**********

    //const vec3 sepia = vec3(94.0, 38.0, 18.0) / 255.0;
    //fxaa_color = mix(fxaa_color, sepia, 0.1); // * luma_mod;


    // these were the original (and less than popular) settings
    //fxaa_color = overlay(fxaa_color, vec3(0.70));	// was 0.75, but Bil suggested a BIT more color
    //fxaa_color = BrightnessSaturationContrast(fxaa_color, 1.5, 0.8, 1.1);

    //vec3 fxaa2_color = BlendSoftLight(fxaa_color, vec3(0.545, 0.27, 0.07));
    
    // Red Thunder default
    //fxaa_color = overlay(fxaa_color, vec3(0.75));
    //fxaa_color = BrightnessSaturationContrast(fxaa_color, 1.1, 0.75, 0.75); //1.1);


    fxaa_color = BrightnessSaturationContrast(fxaa_color, 1.06, 1.00, 1.04); //1.1);

    ///vec3 fxaa_color2 = overlay(fxaa_color, vec3(0.75));
    ///fxaa_color2 = BrightnessSaturationContrast(fxaa_color2, 1.1, 0.4, 0.5); //1.1);
    ///fxaa_color = BlendSoftLight(fxaa_color2, fxaa_color);

//fxaa_color = BrightnessSaturationContrast(fxaa_color, 1.1, 0.75, 0.75);

    // holy crap, the parameters are actually brightness, saturation, contrast
    //fxaa_color = BrightnessSaturationContrast(fxaa_color, 1.0, 0.8, 0.8); //0.8, 1.1, 1.1); //0.75, 0.5, 1.0); //1.05, 0.8, 1.0); //1.7, 0.6); //1.5, 0.8, 1.1);
    gl_FragColor = vec4(fxaa_color, 1.0);

    // a bypass filter (sort of Black Hawk Down-ish) that gives a nice effect
    //gl_FragColor = bypass(gl_FragColor);
    //gl_FragColor.xyz = BrightnessSaturationContrast(gl_FragColor.xyz, 1.0, 0.8, 0.8);
    
    //gl_FragColor = mix(gl_FragColor, vec4(0.0, 0.0, 0.5, 1.0), 0.1);
    //gl_FragColor = mix(gl_FragColor, vec4(139.0, 69.0, 19.0, 255.0) / 255.0, 0.07);

    

    // and a final brightness modifier
    gl_FragColor *= 1.12; //1.05;
    
    // if you want this shader to have no effect, uncomment the following line
    //gl_FragColor.xyz = texture2D(RTScene, pos).xyz;
}